#ifndef _C84_MEM_h_
#define _C84_MEM_h_		/* avoid multiple includes */

/*
 * c84_mem.h -- PIC 16C84 assembler memory definitions,
 *		CSE510 Project, Fall 1995
 *
 * Copyright (C) 1995-6 James Cleverdon
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Address:
 *	James Cleverdon
 *	16465 S.W. Estuary Dr. # 101
 *	Beaverton, OR 97006-7954, USA
 *
 *	jamesc@sequent.com
 *
 * $Log:	c84_mem.h,v $
 * Revision 1.0  95/12/19  01:28:34  jamesc
 * "CSE510_project"
 * 
 */

#ident "$Id: c84_mem.h,v 1.0 95/12/19 01:28:34 jamesc Exp Locker: jamesc $"

/* just in case the compiler doesn't come with ANSI typedefs */

#ifndef uchar
#define uchar		unsigned char
#define ushort		unsigned short
#define uint		unsigned int
#define ulong		unsigned long
#endif


/*
 * Defines for PIC16C84 microcontroller chip.
 *
 * The PIC 16C84 RISC microcontroller chip has four (count 'em -- 4) different
 * memory spaces:
 *
 * 1) Program memory:  1K of 14-bit words of EEROM.  Can only be altered in
 *	programming mode.  Can only be executed by programs, not referenced.
 *	Hence the RETLW hack to put tables in program memory.
 *
 * 2) Data file memory:  36 bytes of real RAM, plus some special function
 *	registers that may have available bits.  Data memory is directly
 *	addressable in pages, bit addressable, and indirectly addressable
 *	through the FSR.
 *
 * 3) EEROM data memory:  64 bytes of EEROM that may be reprogrammed while
 *	the 16C84 is running (yes, even at 2.0 V for the LC version).  It
 *	is accessed through the EEADR and EEDATA file registers by programs
 *	and is made part of configuration memory for programming purposes.
 *
 * 4) Configuration memory:  Accessible only in programming mode, this memory
 *	space contains the configuration fuse word and four 14-bit ID words of
 *	EEROM.  The ID words are available for use to hold a checksum, or a
 *	version number, or some other identifing string of bytes.  If the
 *	code-protection fuse is set, then only the low 7 bits of the ID words
 *	are usable -- the high 7 bits must be all 1s.
 *
 *	Also, the EEROM appears in this space when programming.
 */

/* Memory section maximum sizes. */

#define CODE_MAX	0x3FF			/* last word of code space */
#define DATA_MAX	0x1FF			/* last byte of data space */
#define EEDATA_MAX	0x3F			/* last byte of EEROM	*/
#define ID_MAX		0x3			/* last word of ID EEROM */

/* first available word or byte in each memory space */

#define CODE_ORIGIN	0x0			/* code: 000 - 3FF	*/
#define DATA_ORIGIN	0xC			/* reg: 0 - B, RAM: C - 2f */
#define EEDATA_ORIGIN	0x0			/* EEROM: 00 - 3F	*/
#define CONFIG_ORIGIN	0x2000			/* config: 0x2000 - 0x2007 */
#define ID_ORIGIN	0x0			/* id: 0 - 3		*/

/* data space is accessed by pages -- 16C84 page limits */

#define FILE_MAX	0x7F			/* max file reg in opcode */
#define FILE_MIN_P0	0x0			/* min files in page 0	*/
#define FILE_MAX_P0	0x2F			/* max files in page 0	*/
#define FILE_MIN_P1	0x80			/* min files in page 1	*/
#define FILE_MAX_P1	0xAF			/* max files in page 1	*/

/* programming mode defines and addresses */

#define CODE_START	0x0000			/* code start address	*/
#define CODE_END	0x03FF			/* code end address	*/
#define ID_START	0x2000			/* ID EEROM start address */
#define ID_END		0x2003			/* ID EEROM end address	*/
#define FUSE_ADDR	0x2007			/* config fuse address	*/
#define EEDATA_START	0x2100			/* EE data start address*/
#define EEDATA_END	0x213F			/* EE data end address	*/

#endif /* _C84_MEM_h_ */
