
%{

/*
 * gram.y -- YACC grammar for 16C84 assembler, CSE510 Project, Fall 1995
 *
 * Copyright (C) 1995-6 James Cleverdon
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Address:
 *	James Cleverdon
 *	16465 S.W. Estuary Dr. # 101
 *	Beaverton, OR 97006-7954, USA
 *
 *	jamesc@sequent.com
 *
 * $Log:	gram.y,v $
 * Revision 1.1  96/01/03  11:50:29  jamesc
 * Added code to insist that opt_label be on
 * column 0 to fix bug where a misspelled
 * opcode is treated as a label and silently
 * eaten.  Thanks to adrian@fangorn.demon.co.uk
 * for pointing this out!  Also wrapped long
 * action lines to 80 columns.
 * 
 * Revision 1.0  1995/12/19 01:28:34  jamesc
 * "CSE510_project"
 *
 */

#ident "$Id: gram.y,v 1.1 96/01/03 11:50:29 jamesc Exp Locker: jamesc $"

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "c84.h"
#include "c84_ext.h"

%}

	/* opcodes */

%token	T_ADDLW		1
%token	T_ADDWF		2
%token	T_ANDLW		3
%token	T_ANDWF		4
%token	T_BCF		5
%token	T_BSF		6
%token	T_BTFSC		7
%token	T_BTFSS		8
%token	T_CALL		9	/* 10 is \n */
%token	T_CLRF		11
%token	T_CLRW		12
%token	T_CLRWDT	13
%token	T_COMF		14
%token	T_DECF		15
%token	T_DECFSZ	16
%token	T_GOTO		17
%token	T_INCF		18
%token	T_INCFSZ	19
%token	T_IORLW		20
%token	T_IORWF		21
%token	T_MOVF		22
%token	T_MOVLW		23
%token	T_MOVWF		24
%token	T_NOP		25
%token	T_OPTION	26
%token	T_RETFIE	27
%token	T_RETLW		28
%token	T_RETURN	29
%token	T_RLF		30
%token	T_RRF		31
%token	T_SLEEP		32
%token	T_SUBLW		65
%token	T_SUBWF		66
%token	T_SWAPF		67
%token	T_TRIS		68
%token	T_XORLW		69
%token	T_XORWF		70

	/* psuedo-ops and assembler directives */

%token	T_BIT		71
%token	T_DBYTE		72
%token	T_DORG		73
%token	T_DWORD		74
%token	T_EBYTE		75
%token	T_END		76
%token	T_EQU		77
%token	T_EORG		78
%token	T_EWORD		79
%token	T_FILL		80
%token	T_FUSE		81
%token	T_ID		35
%token	T_ORG		83
%token	T_UNEQU		84
%token	'='		/* same as T_EQU */

	/* constant tokens */

%token	T_B_CONST	85	T_CONST		86
%token	T_LABEL		87	T_UNDEF_LABEL	88
%token	T_STRING	89
%token	'$'

%token	'\n'

%left	','
%left	T_LOG_OR	90
%left	T_LOG_AND	91
%left	'|'
%left	'^'
%left	'&'
%left	T_LOG_EQ	92	T_LOG_NE	93
%left	'>'			T_LOG_GE	97	'<'	T_LOG_LE  98
%left	T_LSHIFT	99	T_RSHIFT	100
%left	'+'		'-'
%left	'*'		'/'	'%'
%right	T_MAX		101	T_MIN		102
%right	T_HIGH		103	T_LOW		104
%right	'!'		'~'
%nonassoc T_UMINUS	105
%token	'('		')'

%start	program

%%

	/* a program is 0 or more lines */
program	:				{ $$ = NNULL; }
	| program line
	;

	/* line is a opt label, an op or psuedo-op, ending in nl */
line	: opt_label '\n'		{
		$$ = NNULL;
		if ($1) {
			if ($1->n_col != 0) {
				nerr($1, "Not a known opcode!");
			}
			label_def($1, -1, T_CONST);
		}
					}
	| opt_label op '\n'		{ $$ = c_gen_op($1, $2); }
	| opt_label pseudo '\n'		{
		$$ = NNULL;
		if ($1) {
			label_def($1, -1, T_CONST);
		}
					}
	| label T_BIT f_b '\n'		{ $$ = c_bit($1, $2, $3); }
	| opt_label T_DBYTE expr '\n'	{ $$ = c_dbyte($1, $2, $3); }
	| opt_label T_DORG expr '\n'	{ $$ = c_dorg($1, $2, $3); }
	| opt_label T_DWORD expr '\n'	{ $$ = c_dword($1, $2, $3); }
	| opt_label T_EBYTE expr_list '\n' { $$ = c_ebyte($1, $2); }
	| opt_label T_EORG expr '\n'	{ $$ = c_eorg($1, $2, $3); }
	| opt_label T_EWORD expr_list '\n' { $$ = c_eword($1, $2); }
	| label T_EQU expr '\n'		{ $$ = c_equ($1, $2, $3); }
	| label '=' expr '\n'		{ $$ = c_equ($1, $2, $3); }
	| opt_label T_FILL expr ',' expr '\n' {
		$$ = c_fill($1, $2, $3, $4, $5);
					}
	| opt_label T_ORG expr '\n'	{ $$ = c_org($1, $2, $3); }
	| opt_label T_RETLW expr_list '\n' { $$ = c_retlw($1, $2); }
	;

	/* expression list: comma separated constant values */
expr_list : expr_str			{ $$ = un_op($0, $1); }
	| expr_list ',' expr_str	{ $$ = comma_list($0, $2, $3); }
	;

	/* expression, label, or string */
expr_str : expr
	| T_STRING
	;

	/* psuedo-ops and assembler directives */
pseudo	: T_FUSE expr			{ $$ = c_fuse($1, $2); }
	| T_ID expr_list		{ $$ = c_id($1); }
	| T_UNEQU expr			{ $$ = c_unequ($1, $2); }
	| T_END				{
		$$ = NNULL;
		free_node($1);
		YYACCEPT;
					}
	;

	/* optional label at start of line */
opt_label :				{ $$ = NNULL; }
	| label
	;

	/* a label is either a known label, or a undefined label */
label	: T_CONST
	| T_UNDEF_LABEL
	;

	/* opcode list */
op	: T_ADDLW expr			{ $$ = un_op($1, $2); }
	| T_ADDWF f_d			{ $$ = un_op($1, $2); }
	| T_ANDLW expr			{ $$ = un_op($1, $2); }
	| T_ANDWF f_d			{ $$ = un_op($1, $2); }
	| T_BCF f_b			{ $$ = un_op($1, $2); }
	| T_BSF f_b			{ $$ = un_op($1, $2); }
	| T_BTFSC f_b			{ $$ = un_op($1, $2); }
	| T_BTFSS f_b			{ $$ = un_op($1, $2); }
	| T_CALL label			{ $$ = un_op($1, $2); }
	| T_CLRF expr			{ $$ = un_op($1, $2); }
	| T_CLRW
	| T_CLRWDT
	| T_COMF f_d			{ $$ = un_op($1, $2); }
	| T_DECF f_d			{ $$ = un_op($1, $2); }
	| T_DECFSZ f_d			{ $$ = un_op($1, $2); }
	| T_GOTO label			{ $$ = un_op($1, $2); }
	| T_INCF f_d			{ $$ = un_op($1, $2); }
	| T_INCFSZ f_d			{ $$ = un_op($1, $2); }
	| T_IORLW expr			{ $$ = un_op($1, $2); }
	| T_IORWF f_d			{ $$ = un_op($1, $2); }
	| T_MOVF f_d			{ $$ = un_op($1, $2); }
	| T_MOVLW expr			{ $$ = un_op($1, $2); }
	| T_MOVWF expr			{ $$ = un_op($1, $2); }
	| T_NOP
	| T_OPTION
	| T_RETFIE
	| T_RETURN
	| T_RLF f_d			{ $$ = un_op($1, $2); }
	| T_RRF f_d			{ $$ = un_op($1, $2); }
	| T_SLEEP
	| T_SUBLW expr			{ $$ = un_op($1, $2); }
	| T_SUBWF f_d			{ $$ = un_op($1, $2); }
	| T_SWAPF f_d			{ $$ = un_op($1, $2); }
	| T_TRIS expr			{ $$ = un_op($1, $2); }
	| T_XORLW expr			{ $$ = un_op($1, $2); }
	| T_XORWF f_d			{ $$ = un_op($1, $2); }
	;

	/* argument: file, bit */
f_b	: expr ',' expr			{ $$ = c_f_b($1, $2, $3); }
	| T_B_CONST
	;

	/* argument: file, dest (note: lousy syntax, but dest is optional) */
f_d	: expr ',' expr			{ $$ = c_f_d($1, $2, $3); }
	| expr				{ $$ = c_f_d($1, NNULL, NNULL); }
	;

	/* expressions */
			/* OPTION is both a keyword and a register name */
expr	: T_OPTION			{ $$ = label_scan("OPTION"); }
	| T_CONST
	| T_B_CONST
	| T_UNDEF_LABEL
	| '$'				{ $$ = c_dol($1); }
	| '(' expr ')'			{ $$ = un_op($1, $2); free_node($3); }
	| '-' expr %prec T_UMINUS	{
			$1->n_tok = T_UMINUS;
			$$ = un_op($1, $2);
					}
	| '~' expr			{ $$ = un_op($1, $2); }
	| '!' expr			{ $$ = un_op($1, $2); }
	| T_HIGH '(' expr ')'		{
		free_node($2);
		free_node($4);
		$$ = un_op($1, $3);
					}
	| T_LOW '(' expr ')'		{
		free_node($2);
		free_node($4);
		$$ = un_op($1, $3);
					}
	| T_MAX '(' expr ',' expr ')'	{
		free_node($2);
		free_node($4);
		free_node($6);
		$$ = bin_op($3, $1, $5);
					}
	| T_MIN '(' expr ',' expr ')'	{
		free_node($2);
		free_node($4);
		free_node($6);
		$$ = bin_op($3, $1, $5);
					}
	| expr '*' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '/' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '%' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '+' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '-' expr			{ $$ = bin_op($1, $2, $3); }
	| expr T_LSHIFT expr		{ $$ = bin_op($1, $2, $3); }
	| expr T_RSHIFT expr		{ $$ = bin_op($1, $2, $3); }
	| expr '&' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '^' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '|' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '<' expr			{ $$ = bin_op($1, $2, $3); }
	| expr '>' expr			{ $$ = bin_op($1, $2, $3); }
	| expr T_LOG_EQ expr		{ $$ = bin_op($1, $2, $3); }
	| expr T_LOG_NE expr		{ $$ = bin_op($1, $2, $3); }
	| expr T_LOG_LE expr		{ $$ = bin_op($1, $2, $3); }
	| expr T_LOG_GE expr		{ $$ = bin_op($1, $2, $3); }
	| expr T_LOG_AND expr		{ $$ = bin_op($1, $2, $3); }
	| expr T_LOG_OR expr		{ $$ = bin_op($1, $2, $3); }
	;

%%

/*+++++ End of gram.y +++++*/

